@@ -1,4 +1,4 @@
-require 'sixarm_ruby_unaccent'
+# frozen_string_literal: true
 
 require 'countries/version'
 
