@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module ISO3166
   class << self
     attr_writer :configuration
@@ -17,13 +19,24 @@ module ISO3166
   end
 
   class Configuration
-    attr_accessor :locales, :loaded_locales
+    attr_accessor :loaded_locales
 
     def initialize
-      @locales = default_locales
       @loaded_locales = []
     end
 
+    def locales
+      @locales ||= default_locales
+    end
+
+    def locales=(value)
+      if value.is_a?(Array)
+        @locales = value.map(&:to_sym).uniq
+      else
+        raise ArgumentError, "Expected an Array of locales, got #{value.class}"
+      end
+    end
+
     # Enables the integration with the {Money}[https://github.com/RubyMoney/money] gem
     #
     # Please note that it requires you to add "money" gem to your gemfile.
@@ -44,9 +57,10 @@ module ISO3166
 
     private
 
+    # :reek:UtilityFunction
     def default_locales
       locales = if Object.const_defined?('I18n') && I18n.respond_to?(:available_locales)
-                  I18n.available_locales
+                  I18n.available_locales.dup
                 else
                   [:en]
                 end
