@@ -1,16 +1,22 @@
+# frozen_string_literal: true
+
 module ISO3166
+  # :reek:TooManyMethods
   class Country
     extend CountryClassMethods
+    extend ConversionMethods
+    extend CountryFinderMethods
     include Emoji
+    include CountrySubdivisionMethods
     attr_reader :data
 
-    ISO3166::DEFAULT_COUNTRY_HASH.each do |method_name, _type|
+    ISO3166::DEFAULT_COUNTRY_HASH.each_key do |method_name|
       define_method method_name do
         data[method_name.to_s]
       end
     end
 
-    ISO3166::DEFAULT_COUNTRY_HASH['geo'].each do |method_name, _type|
+    ISO3166::DEFAULT_COUNTRY_HASH['geo'].each_key do |method_name|
       define_method method_name do
         data['geo'][method_name.to_s]
       end
@@ -21,6 +27,7 @@ module ISO3166
       reload
     end
 
+    # :reek:NilCheck
     def valid?
       !(data.nil? || data.empty?)
     end
@@ -31,24 +38,8 @@ module ISO3166
     alias zip_format postal_code_format
     alias languages languages_official
 
-    def latitude_dec
-      if RUBY_VERSION =~ /^3\.\d\.\d/
-        warn "DEPRECATION WARNING: The Country#latitude_dec method has been deprecated and will be removed in 5.0. Please use Country#latitude instead.", uplevel: 1, category: :deprecated
-      else
-        warn "DEPRECATION WARNING: The Country#latitude_dec method has been deprecated and will be removed in 5.0. Please use Country#latitude instead.", uplevel: 1
-      end
-      latitude
-    end
-
-    def longitude_dec
-      if RUBY_VERSION =~ /^3\.\d\.\d/
-        warn "DEPRECATION WARNING: The Country#longitude_dec method has been deprecated and will be removed in 5.0. Please use Country#longitude instead.", uplevel: 1, category: :deprecated
-      else
-        warn "DEPRECATION WARNING: The Country#longitude_dec method has been deprecated and will be removed in 5.0. Please use Country#longitude instead.", uplevel: 1
-      end
-      longitude
-    end
-
+    # :reek:FeatureEnvy
+    # :reek:ManualDispatch
     def ==(other)
       other.respond_to?(:alpha2) && other.alpha2 == alpha2
     end
@@ -65,56 +56,87 @@ module ISO3166
       to_s <=> other.to_s
     end
 
-    #  +true+ if this Country has any Subdivisions.
-    def subdivisions?
-      !subdivisions.empty?
+    # Access country data by key, symbol or string.
+    # :reek:ManualDispatch
+    def [](key)
+      key = key.to_s if key.is_a?(Symbol)
+      data[key] || send(key)
     end
 
-    # @return [Array<ISO3166::Subdivision>] the list of subdivisions for this Country.
-    def subdivisions
-      @subdivisions ||= if data['subdivisions']
-                          self.class.create_subdivisions(data['subdivisions'])
-                        else
-                          self.class.subdivisions(alpha2)
-                        end
-    end
-
-    # @param locale [String] The locale to use for translations.
-    # @return [Array<Array>] This Country's subdivision pairs of names and codes.
-    def subdivision_names_with_codes(locale = 'en')
-      subdivisions.map { |k, v| [v.translations[locale] || v.name, k] }
-    end
-
-    alias states subdivisions
-
     # +true+ if this country is a member of the European Union.
+    # :reek:NilCheck :reek:DuplicateMethodCall
     def in_eu?
       data['eu_member'].nil? ? false : data['eu_member']
     end
 
+    # +true+ if this country is a member of the G7.
+    # :reek:NilCheck :reek:DuplicateMethodCall
+    def in_g7?
+      data['g7_member'].nil? ? false : data['g7_member']
+    end
+
+    # +true+ if this country is a member of the G20.
+    # :reek:NilCheck :reek:DuplicateMethodCall
+    def in_g20?
+      data['g20_member'].nil? ? false : data['g20_member']
+    end
+
+    # +true+ if this country is a member of the European Economic Area or it is UK
+    def gdpr_compliant?
+      data['eea_member'] || alpha2 == 'GB'
+    end
+
     # +true+ if this country is a member of the European Economic Area.
+    # :reek:NilCheck :reek:DuplicateMethodCall
     def in_eea?
       data['eea_member'].nil? ? false : data['eea_member']
     end
 
     # +true+ if this country is a member of the European Single Market.
+    # :reek:NilCheck :reek:DuplicateMethodCall
     def in_esm?
       data['esm_member'].nil? ? in_eea? : data['esm_member']
     end
 
+    # +true+ if this country is a member of the EU VAT Area.
+    # :reek:NilCheck :reek:DuplicateMethodCall
+    def in_eu_vat?
+      data['euvat_member'].nil? ? in_eu? : data['euvat_member']
+    end
+
+    # +true+ if this country is a member of the United Nations.
+    # :reek:NilCheck :reek:DuplicateMethodCall
+    def in_un?
+      data['un_member'].nil? ? false : data['un_member']
+    end
+
+    # @return [String] The ISO 3166-1 "Short name lower case" value for this Country.
+    def iso_short_name_lower_case
+      data['iso_short_name_lower_case'].nil? ? data['iso_short_name'] : data['iso_short_name_lower_case']
+    end
+
+    # @return [String] The regex for valid postal codes in this Country
+    def postal_code_format
+      "\\A#{data['postal_code_format']}\\Z" if postal_code
+    end
+
     def to_s
       data['iso_short_name']
     end
 
     # @return [Array<String>] the list of names for this Country in all loaded locales.
     def translated_names
-      data['translations'].values
+      data['translations'].values.compact
     end
 
     # @param locale [String] The locale to use for translations.
     # @return [String] the name of this Country in the selected locale.
-    def translation(locale = 'en')
-      data['translations'][locale.to_s.downcase]
+    # :reek:FeatureEnvy
+    def translation(locale = :en)
+      locale = locale.to_sym if locale.is_a?(String)
+      locale = locale.downcase if locale.match?(/[A-Z]/)
+
+      data['translations'][locale]
     end
 
     # @return [String] the “common name” of this Country in English.
@@ -123,7 +145,7 @@ module ISO3166
       translation('en')
     end
 
-    # @return [Array<String>] TThe list of names for this Country, in this Country's locales.
+    # @return [Array<String>] The list of names for this Country, in this Country's locales.
     def local_names
       ISO3166.configuration.locales = (ISO3166.configuration.locales + languages.map(&:to_sym)).uniq
       reload
@@ -136,28 +158,6 @@ module ISO3166
       @local_name ||= local_names.first
     end
 
-    # @return [String] This Country's ISO Short Name
-    # @deprecated Use {#iso_short_name} instead.
-    def name
-      if RUBY_VERSION =~ /^3\.\d\.\d/
-        warn "DEPRECATION WARNING: The Country#name method has been deprecated. Please use Country#iso_short_name instead or refer to the README file for more information on this change.", uplevel: 1, category: :deprecated
-      else
-        warn "DEPRECATION WARNING: The Country#name method has been deprecated. Please use Country#iso_short_name instead or refer to the README file for more information on this change.", uplevel: 1
-      end
-      iso_short_name
-    end
-
-    # @return [Array<String>] Array of unofficial, slang names or aliases for this Country
-    # @deprecated Use {#unofficial_names} instead.
-    def names
-      if RUBY_VERSION =~ /^3\.\d\.\d/
-        warn "DEPRECATION WARNING: The Country#names method has been deprecated. Please use Country#unofficial_names instead or refer to the README file for more information on this change.", uplevel: 1, category: :deprecated
-      else
-        warn "DEPRECATION WARNING: The Country#names method has been deprecated. Please use Country#unofficial_names instead or refer to the README file for more information on this change.", uplevel: 1
-      end
-      unofficial_names
-    end
-
     # @!attribute alpha2
     #   @return [String] the ISO3166 alpha-2 code for this Country
     #
@@ -176,6 +176,9 @@ module ISO3166
     # @!attribute currency_code
     #   @return [String] the ISO 4217 currency code for this Country
     #
+    # @!attribute distance_unit
+    #   @return [String] the unit for roading distance and speed for this Country
+    #
     # @!attribute gec
     #   @return [String] the "Geopolitical Entities and Codes", formerly FIPS 10-4 code for this Country
     #
@@ -218,9 +221,6 @@ module ISO3166
     # @!attribute postal_code
     #   @return [Boolean] Does this Country uses postal codes in addresses
     #
-    # @!attribute postal_code_format
-    #   @return [String] The regex for valid postal codes in this Country
-    #
     # @!attribute region
     #   @return [String] The Region this country is in. Approximately matches the United Nations geoscheme
     #
@@ -239,11 +239,12 @@ module ISO3166
     # @!attribute vat_rates
     #   @return [Hash] the hash of VAT Rates for this Country
     #
+    # @!attribute vehicle_registration_code
+    #   @return [String] The vehicle registration code for this Country
+    #
     # @!attribute world_region
     #   @return [String] The "World Region" this country is in: +"AMER"+ , +"APAC"+ or +"EMEA"+
 
-
-
     private
 
     def reload
