@@ -1,8 +1,11 @@
+# frozen_string_literal: true
+
 module Sources
   module CLDR
     # Auxiliary Subdivision class to support loading Unicode CLDR data to update local files
     class Subdivision
       attr_reader :xml, :language_code
+
       def initialize(language_code:, xml:)
         @language_code = language_code
         @xml = xml
@@ -17,7 +20,7 @@ module Sources
       end
 
       def code
-        type[2..-1].upcase
+        type[2..].upcase
       end
 
       def type
