@@ -1,11 +1,20 @@
-# encoding: utf-8
+# frozen_string_literal: true
 
 require 'spec_helper'
 NUM_OF_COUNTRIES = 249
 describe ISO3166::Country do
-  before { ISO3166.configuration.enable_currency_extension! }
+  before do
+    ISO3166.configuration.locales = %i[en]
+    ISO3166::Data.reset
+    ISO3166.configuration.enable_currency_extension!
+  end
+
   let(:country) { ISO3166::Country.search('US') }
 
+  it 'handles respond_to_missing values' do
+    expect(described_class.respond_to?(:arr)).not_to be_nil
+  end
+
   it 'allows to create a country object from a symbol representation of the alpha2 code' do
     country = described_class.new(:us)
     expect(country.data).not_to be_nil
@@ -47,13 +56,19 @@ describe ISO3166::Country do
     expect(country.iso_short_name).to eq('United States of America')
   end
 
-  it 'should return alternate names' do
-    expect(country.unofficial_names).to eq(['United States', 'USA', 'Murica', 'Vereinigte Staaten von Amerika', 'États-Unis', 'Estados Unidos', 'アメリカ合衆国', 'Verenigde Staten'])
+  it 'should return 3166 iso_short_name_lower_case if available' do
+    expect(country.iso_short_name_lower_case).to eq('United States of America (the)')
+    portugal = ISO3166::Country.new('PT')
+    expect(portugal.iso_short_name_lower_case).to eq('Portugal')
   end
 
-  it 'should return translations' do
-    expect(country.translations).to be
-    expect(country.translations['en']).to eq('United States')
+  it 'should return alternate names' do
+    expect(country.unofficial_names).to eq(['United States',
+                                            'United States of America', 'USA',
+                                            'Vereinigte Staaten von Amerika',
+                                            'États-Unis', 'Estados Unidos',
+                                            'アメリカ合衆国', 'Verenigde Staten',
+                                            'Соединенные Штаты Америки'])
   end
 
   it 'should return latitude' do
@@ -72,6 +87,10 @@ describe ISO3166::Country do
     expect(country.continent).to eq('North America')
   end
 
+  it 'should return distance unit' do
+    expect(country.distance_unit).to eq('MI')
+  end
+
   it 'knows about whether or not the country uses postal codes' do
     expect(country.zip).to be_truthy
   end
@@ -85,7 +104,8 @@ describe ISO3166::Country do
     expect(country.postal_code_format).to_not be_nil
 
     regex = Regexp.new(country.postal_code_format)
-    expect(regex).to match("12345-6789")
+    expect(regex).to match('12345-6789')
+    expect(regex).not_to match('12345-67890')
 
     antarctica = ISO3166::Country.search('AQ')
     expect(antarctica.postal_code_format).to be_nil
@@ -99,16 +119,28 @@ describe ISO3166::Country do
     expect(country.subregion).to eq('Northern America')
   end
 
+  it 'should return vehicle registration code' do
+    expect(country.vehicle_registration_code).to eq('USA')
+  end
+
   it 'should return world region' do
     expect(country.world_region).to eq('AMER')
   end
 
-  context 'with Turkey' do
+  context 'with Türkiye' do
     let(:country) { ISO3166::Country.search('TR') }
 
     it 'should indicate EMEA as the world region' do
       expect(country.world_region).to eq('EMEA')
     end
+
+    it 'has iso_short_name Türkiye' do
+      expect(country.iso_short_name).to eq('Türkiye')
+    end
+
+    it 'has iso_long_name Republic of Türkiye' do
+      expect(country.iso_long_name).to eq('The Republic of Türkiye')
+    end
   end
 
   context 'with Japan' do
@@ -123,7 +155,7 @@ describe ISO3166::Country do
     let(:country) { ISO3166::Country.search('BE') }
 
     it 'should return its local names based on its languages' do
-      expect(country.local_names).to match_array(%w(België Belgique Belgien))
+      expect(country.local_names).to match_array(%w[België Belgique Belgien])
     end
 
     it 'should return its first local name' do
@@ -137,7 +169,7 @@ describe ISO3166::Country do
         ISO3166::Data.register(
           alpha2: 'BR2',
           iso_short_name: 'Brazil',
-          languages_official: %w(pt-BR),
+          languages_official: %w[pt-BR],
           translations: {
             'pt-BR' => 'Translation for pt-BR',
             'pt' => 'Translation for pt'
@@ -161,7 +193,7 @@ describe ISO3166::Country do
         ISO3166::Data.register(
           alpha2: 'BR2',
           iso_short_name: 'Brazil',
-          languages_official: %w(pt-BR),
+          languages_official: %w[pt-BR],
           translations: {
             'pt' => 'Translation for pt'
           }
@@ -219,7 +251,7 @@ describe ISO3166::Country do
   end
 
   describe 'subdivisions' do
-    let(:virginia) { country.states['VA'] }
+    let(:virginia) { country.subdivisions['VA'] }
     it 'should return an empty hash for a country with no ISO3166-2' do
       expect(ISO3166::Country.search('VA').subdivisions.size).to eq(0)
     end
@@ -228,10 +260,6 @@ describe ISO3166::Country do
       expect(country.subdivisions.size).to eq(57)
     end
 
-    it 'should be available through states' do
-      expect(country.states.size).to eq(57)
-    end
-
     it 'should be a subdivision object' do
       expect(virginia).to be_a(ISO3166::Subdivision)
     end
@@ -249,6 +277,60 @@ describe ISO3166::Country do
     end
   end
 
+  describe 'subdivision_types' do
+    it 'should return an array of subdivision types' do
+      expect(country.subdivision_types).to contain_exactly('district', 'state', 'outlying_area')
+    end
+
+    it 'should return an array of subdivision types even if there is only a single type' do
+      expect(ISO3166::Country['LI'].subdivision_types).to contain_exactly('commune')
+    end
+
+    it 'should return an empty array if country has no subdivisions' do
+      expect(ISO3166::Country['AS'].subdivisions?).to be_falsey
+      expect(ISO3166::Country['AS'].subdivision_types).to eq([])
+    end
+  end
+
+  describe 'subdivisions_of_types' do
+    it 'given a single type, should return an array of subdivisions that match the type' do
+      us_states = country.subdivisions_of_types(%w[state])
+      expect(us_states.size).to eq(50)
+      dc = country.subdivisions_of_types(%w[district])
+      expect(dc.size).to eq(1)
+    end
+
+    it 'given multiple types, should return an array of subdivisions matching the types' do
+      us_states_plus_dc = country.subdivisions_of_types(%w[state district])
+      expect(us_states_plus_dc.size).to eq(51)
+    end
+
+    it 'given multiple types where at least one does not exist for that country, should work without issue' do
+      us_states_plus_dc = country.subdivisions_of_types(%w[state district governorate])
+      expect(us_states_plus_dc.size).to eq(51)
+    end
+
+    it 'given only types that do not exist for that country, should return an empty collection' do
+      should_be_empty = ISO3166::Country['PT'].subdivisions_of_types(%w[state county])
+      expect(should_be_empty).to be_empty
+    end
+  end
+
+  describe 'humanized_subdivision_types' do
+    it 'should return an array of humanized subdivision types' do
+      expect(country.humanized_subdivision_types).to contain_exactly('District', 'State', 'Outlying area')
+    end
+
+    it 'should return an array of subdivision types even if there is only a single type' do
+      expect(ISO3166::Country['LI'].humanized_subdivision_types).to contain_exactly('Commune')
+    end
+
+    it 'should return an empty array if country has no subdivisions' do
+      expect(ISO3166::Country['AS'].subdivisions?).to be_falsey
+      expect(ISO3166::Country['AS'].humanized_subdivision_types).to eq([])
+    end
+  end
+
   describe 'subdivision_names_with_codes' do
     it 'should return an alphabetized list of all subdivisions names with codes' do
       subdivisions = ISO3166::Country.search('EG').subdivision_names_with_codes
@@ -259,16 +341,60 @@ describe ISO3166::Country do
     end
 
     it 'should return an alphabetized list of subdivision names translated to current locale with codes' do
-      ISO3166.configuration.locales = [:es, :de, :en]
+      ISO3166.configuration.locales = %i[es de en]
+      ISO3166::Data.reset
 
       subdivisions = ISO3166::Country.search('EG').subdivision_names_with_codes(:es)
       expect(subdivisions).to be_an(Array)
       expect(subdivisions.first[0]).to be_a(String)
       expect(subdivisions.size).to eq(27)
+      expect(subdivisions.first[0]).to eq('Alejandría')
+    end
+
+    it 'falls back to the base subdivision name if the translation is missing' do
+      ISO3166.configuration.locales = %i[af]
+      ISO3166::Data.reset
+
+      subdivisions = ISO3166::Country.search('EG').subdivision_names_with_codes(:af)
+      expect(subdivisions).to be_an(Array)
+      expect(subdivisions.first[0]).to be_a(String)
+      expect(subdivisions.size).to eq(27)
       expect(subdivisions.first[0]).to eq('Al Iskandariyah')
     end
   end
 
+  describe 'subdivision_names' do
+    it 'should return an alphabetized list of all subdivisions names' do
+      subdivisions = ISO3166::Country.search('EG').subdivision_names
+      expect(subdivisions).to be_an(Array)
+      expect(subdivisions.first).to be_a(String)
+      expect(subdivisions.size).to eq(27)
+      expect(subdivisions.first).to eq('Alexandria')
+    end
+
+    it 'should return an alphabetized list of subdivision names translated to current locale with codes' do
+      ISO3166.configuration.locales = %i[es de en]
+      ISO3166::Data.reset
+
+      subdivisions = ISO3166::Country.search('EG').subdivision_names(:es)
+      expect(subdivisions).to be_an(Array)
+      expect(subdivisions.first).to be_a(String)
+      expect(subdivisions.size).to eq(27)
+      expect(subdivisions.first).to eq('Alejandría')
+    end
+
+    it 'falls back to the base subdivision name if the translation is missing' do
+      ISO3166.configuration.locales = %i[af]
+      ISO3166::Data.reset
+
+      subdivisions = ISO3166::Country.search('EG').subdivision_names(:af)
+      expect(subdivisions).to be_an(Array)
+      expect(subdivisions.first).to be_a(String)
+      expect(subdivisions.size).to eq(27)
+      expect(subdivisions.first).to eq('Al Iskandariyah')
+    end
+  end
+
   describe 'valid?' do
     it 'should return true if country is valid' do
       expect(ISO3166::Country.new('US')).to be_valid
@@ -325,12 +451,12 @@ describe ISO3166::Country do
 
   describe 'all_translated' do
     it 'should return an alphabetized list of all country names translated to the selected locale' do
-      countries = ISO3166::Country.all_translated('fr')
+      countries = ISO3166::Country.all_translated('pt')
       expect(countries).to be_an(Array)
       expect(countries.first).to be_a(String)
-      expect(countries.first).to eq('Aruba')
+      expect(countries.first).to eq('Andorra')
       # countries missing the desired locale will not be added to the list
-      # so all 250 countries may not be returned, 'fr' returns 249, for example
+      # so all 250 countries may not be returned, 'pt' returns 249, for example
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
     end
 
@@ -338,7 +464,7 @@ describe ISO3166::Country do
       countries = ISO3166::Country.all_translated
       expect(countries).to be_an(Array)
       expect(countries.first).to be_a(String)
-      expect(countries.first).to eq('Aruba')
+      expect(countries.first).to eq('Andorra')
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
     end
 
@@ -371,11 +497,11 @@ describe ISO3166::Country do
       expect(countries.first[0]).to be_a(String)
       expect(countries.first[0]).to eq('Afghanistan')
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
-      expect(countries.any?{|pair| !pair[0].html_safe?}).to eq(false)
+      expect(countries.any? { |pair| !pair[0].html_safe? }).to eq(false)
     end
 
     it 'should return an alphabetized list of all country names translated to current locale with ISOCODE alpha2' do
-      ISO3166.configuration.locales = [:es, :de, :en]
+      ISO3166.configuration.locales = %i[es de en]
 
       countries = ISO3166::Country.all_names_with_codes(:es)
       expect(countries).to be_an(Array)
@@ -395,7 +521,7 @@ describe ISO3166::Country do
     end
 
     it 'should return an alphabetized list of all country names translated to current locale with ISOCODE alpha2' do
-      ISO3166.configuration.locales = [:es, :de, :en]
+      ISO3166.configuration.locales = %i[es de en]
 
       countries = ISO3166::Country.all_names_with_codes(:es)
       expect(countries).to be_an(Array)
@@ -407,7 +533,7 @@ describe ISO3166::Country do
 
   describe 'translation' do
     it 'should return the localized name for a country to the selected locale' do
-      ISO3166.configuration.locales = [:es, :de, :en]
+      ISO3166.configuration.locales = %i[es de en]
       countries = ISO3166::Country.new(:de).translation('de')
       expect(countries).to be_an(String)
       expect(countries).to eq('Deutschland')
@@ -426,7 +552,7 @@ describe ISO3166::Country do
 
     context 'should return variant locales' do
       it 'should return different value for Chinese variants' do
-        ISO3166.configuration.locales = [:'zh-cn', :'zh-hk', :'zh-tw']
+        ISO3166.configuration.locales = %i[zh-cn zh-hk zh-tw]
         name_cn = ISO3166::Country['TW'].translation('zh-cn')
         name_hk = ISO3166::Country['TW'].translation('zh-hk')
         name_tw = ISO3166::Country['TW'].translation('zh-tw')
@@ -434,9 +560,9 @@ describe ISO3166::Country do
       end
 
       it 'should return different value for Portuguese variants' do
-        ISO3166.configuration.locales = [:pt, :'pt-br']
-        name_pt = ISO3166::Country['SG'].translation('pt')
-        name_br = ISO3166::Country['SG'].translation('pt-br')
+        ISO3166.configuration.locales = %i[pt pt-br]
+        name_pt = ISO3166::Country['BY'].translation('pt')
+        name_br = ISO3166::Country['BY'].translation('pt-br')
         expect([name_pt, name_br].uniq.size).to eql 2
       end
     end
@@ -444,20 +570,40 @@ describe ISO3166::Country do
 
   describe 'translations' do
     it 'should return an hash of all country names translated to the selected locale' do
-      countries = ISO3166::Country.translations('fr')
+      countries = ISO3166::Country.translations('pt')
       expect(countries).to be_an(Hash)
-      expect(countries.first[0]).to eq('AW')
-      expect(countries.first).to eq(%w(AW Aruba))
+      expect(countries.first[0]).to eq('AD')
+      expect(countries.first).to eq(%w[AD Andorra])
       # countries missing the desired locale will not be added to the list
-      # so all 250 countries may not be returned, 'fr' returns 249, for example
+      # so all 250 countries may not be returned, 'pt' returns 249, for example
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
     end
 
     it 'should return an hash of all country names in English if no locale is passed' do
       countries = ISO3166::Country.translations
       expect(countries).to be_an(Hash)
-      expect(countries.first[0]).to eq('AW')
-      expect(countries.first).to eq(%w(AW Aruba))
+      expect(countries.first[0]).to eq('AD')
+      expect(countries.first).to eq(%w[AD Andorra])
+      expect(countries.size).to eq(NUM_OF_COUNTRIES)
+    end
+
+    it 'should return an hash of all country names translated to the selected locale when locale is a symbol' do
+      countries = ISO3166::Country.translations(:pt)
+      expect(countries).to be_an(Hash)
+      expect(countries.first[0]).to eq('AD')
+      expect(countries.first).to eq(%w[AD Andorra])
+      # countries missing the desired locale will not be added to the list
+      # so all 250 countries may not be returned, 'pt' returns 249, for example
+      expect(countries.size).to eq(NUM_OF_COUNTRIES)
+    end
+
+    it 'should return an hash of all country names translated to the selected locale when locale is symbol with uppercase chars' do
+      countries = ISO3166::Country.translations(:Pt)
+      expect(countries).to be_an(Hash)
+      expect(countries.first[0]).to eq('AD')
+      expect(countries.first).to eq(%w[AD Andorra])
+      # countries missing the desired locale will not be added to the list
+      # so all 250 countries may not be returned, 'pt' returns 249, for example
       expect(countries.size).to eq(NUM_OF_COUNTRIES)
     end
   end
@@ -546,6 +692,12 @@ describe ISO3166::Country do
       expect(spain_data.keys).to eq(['ES'])
     end
 
+    it 'finds country from a subdivision name' do
+      gb_data = ISO3166::Country.find_all_by(:subdivision_names, 'Scotland')
+      expect(gb_data).to be_a Hash
+      expect(gb_data.keys).to eq(['GB'])
+    end
+
     it 'performs reasonably' do
       start = Time.now
       250.times do
@@ -560,6 +712,40 @@ describe ISO3166::Country do
     end
   end
 
+  describe 'conversion methods' do
+    describe 'from_alpha2_to_alpha3' do
+      subject { ISO3166::Country.from_alpha2_to_alpha3(alpha2) }
+
+      context 'when country exists' do
+        let(:alpha2) { 'US' }
+
+        it { is_expected.to eq('USA') }
+      end
+
+      context 'when country does not exist' do
+        let(:alpha2) { '..' }
+
+        it { is_expected.to be_nil }
+      end
+    end
+
+    describe 'from_alpha3_to_alpha2' do
+      subject { ISO3166::Country.from_alpha3_to_alpha2(alpha3) }
+
+      context 'when country exists' do
+        let(:alpha3) { 'USA' }
+
+        it { is_expected.to eq('US') }
+      end
+
+      context 'when country does not exist' do
+        let(:alpha3) { '...' }
+
+        it { is_expected.to be_nil }
+      end
+    end
+  end
+
   describe 'hash finder methods' do
     context "when search name in 'iso_short_name'" do
       subject { ISO3166::Country.find_by_iso_short_name('Poland') }
@@ -579,7 +765,7 @@ describe ISO3166::Country do
       subject { ISO3166::Country.find_by_iso_short_name(country_name) }
 
       context 'with Republic of Korea' do
-        let(:country_name) { 'Korea, Republic of' }
+        let(:country_name) { 'Korea (Republic of)' }
         it 'should return' do
           expect(subject.first).to eq('KR')
         end
@@ -634,6 +820,13 @@ describe ISO3166::Country do
       end
     end
 
+    context "when search name in 'subdivision_names'" do
+      subject { ISO3166::Country.find_by_subdivision_names('Scotland') }
+      it 'should return' do
+        expect(subject.first).to eq('GB')
+      end
+    end
+
     context "when search name in 'translated_names'" do
       before do
         ISO3166.configure do |config|
@@ -648,7 +841,9 @@ describe ISO3166::Country do
 
     context 'when finding by invalid attribute' do
       it 'should raise an error' do
-        expect { ISO3166::Country.find_by_invalid('invalid') }.to raise_error(RuntimeError, "Invalid attribute name 'invalid'")
+        expect { ISO3166::Country.find_by_invalid('invalid') }.to(
+          raise_error(RuntimeError, "Invalid attribute name 'invalid'")
+        )
       end
     end
 
@@ -713,15 +908,99 @@ describe ISO3166::Country do
       end
     end
 
-    context 'regression test for #388' do
-      let(:no_country) { ISO3166::Country.find_country_by_translated_names(nil) }
+    describe '#find_country_by_any_name' do
+      context 'when search name found' do
+        let(:uk) { ISO3166::Country.find_country_by_any_name('United Kingdom') }
+
+        it 'should be a country instance' do
+          expect(uk).to be_a(ISO3166::Country)
+          expect(uk.alpha2).to eq('GB')
+        end
+      end
+
+      context 'when search lowercase name found' do
+        let(:uk) { ISO3166::Country.find_country_by_any_name('united kingdom') }
+
+        it 'should be a country instance' do
+          expect(uk).to be_a(ISO3166::Country)
+          expect(uk.alpha2).to eq('GB')
+        end
+      end
+
+      context 'when the search term contains comma' do
+        let(:korea) { ISO3166::Country.find_country_by_any_name('Korea, Republic of') }
+
+        it 'should be a country instance' do
+          expect(korea).to be_a(ISO3166::Country)
+          expect(korea.alpha2).to eq('KR')
+        end
+      end
+
+      context 'when search translation found' do
+        before do
+          ISO3166.configure do |config|
+            config.locales = [:bs]
+          end
+        end
+        let(:uk) { ISO3166::Country.find_country_by_any_name('Velika Britanija') }
+
+        it 'should be a country instance' do
+          expect(uk).to be_a(ISO3166::Country)
+          expect(uk.alpha2).to eq('GB')
+        end
+      end
+    end
+
+    context 'sanity check for #771' do
+      let(:turkey) { ISO3166::Country.find_country_by_any_name('Turkey') }
 
       it 'should be a country instance' do
+        expect(turkey).to be_a(ISO3166::Country)
+        expect(turkey.alpha2).to eq('TR')
+      end
+    end
+
+    context 'regression test for #746' do
+      let(:no_country) { ISO3166::Country.find_country_by_any_name(nil) }
+
+      it 'should not be a country instance' do
         expect(no_country).to_not be_a(ISO3166::Country)
         expect(no_country).to eq nil
       end
     end
 
+    context 'regression test for #912' do
+      it 'should return nil and not raise an error when searching with nil' do
+        expect(ISO3166::Country[nil]).to be_nil
+      end
+    end
+
+    context 'regression test for #388/#746/#776' do
+      before do
+        ISO3166.configure do |config|
+          config.locales = %i[af am ar as az be bg bn br bs ca cs cy da de dz el en
+                              eo es et eu fa fi fo fr ga gl gu he hi hr hu hy ia id
+                              is it ja ka kk km kn ko ku lt lv mi mk ml mn mr ms mt
+                              nb ne nl nn oc or pa pl ps pt ro ru rw si sk sl so sq
+                              sr sv sw ta te th ti tk tl tr tt ug uk ve vi wa wo xh
+                              zh zu]
+        end
+      end
+
+      let(:no_country) { ISO3166::Country.find_country_by_translated_names(nil) }
+      let(:zimbabwe) { ISO3166::Country['ZW'] }
+
+      it 'should not be a country instance' do
+        expect(no_country).to_not be_a(ISO3166::Country)
+        expect(no_country).to eq nil
+      end
+
+      it 'translated_names should not include nil values' do
+        expect(zimbabwe.translation('no')).to be_nil
+        expect(zimbabwe.translated_names).not_to include(nil)
+      end
+    end
+
     context 'when attempting to search by translations hash' do
       let(:uk) { ISO3166::Country.find_country_by_translations({}) }
 
@@ -729,6 +1008,7 @@ describe ISO3166::Country do
         expect { uk }.to raise_error(RuntimeError)
       end
     end
+
     context 'when search name not found' do
       let(:bogus) { ISO3166::Country.find_country_by_unofficial_names('Does not exist') }
 
@@ -785,8 +1065,9 @@ describe ISO3166::Country do
 
     context 'when finding by invalid attribute' do
       it 'should raise an error' do
-        expect { ISO3166::Country.find_country_by_invalid('invalid') }.to raise_error(RuntimeError,
-                                                                                      "Invalid attribute name 'invalid'")
+        expect { ISO3166::Country.find_country_by_invalid('invalid') }.to(
+          raise_error(RuntimeError, "Invalid attribute name 'invalid'")
+        )
       end
     end
 
@@ -806,6 +1087,19 @@ describe ISO3166::Country do
         expect(country).to be_a(ISO3166::Country)
       end
     end
+
+    context 'regressions for #863' do
+      context 'method_missing' do
+        it 'does not provide a red herring error message' do
+          expect { ISO3166::Country.undefined_method }.to raise_error(NoMethodError) do |error|
+            expect(error.message).to_not match Regexp.new("undefined method `#{Regexp.escape('[]')}' for nil:NilClass")
+          end
+        end
+        it 'provides the expected error message' do
+          expect { ISO3166::Country.undefined }.to raise_error(NoMethodError, /undefined method (`||')undefined'/)
+        end
+      end
+    end
   end
 
   describe 'finder methods respond_to_missing?' do
@@ -847,16 +1141,6 @@ describe ISO3166::Country do
     end
   end
 
-  describe 'names in Data' do
-    it 'should be unique (to allow .find_by_name work properly)' do
-      names = ISO3166::Data.cache.map do |_k, v|
-        [v['iso_short_name'], v['unofficial_names']].flatten.uniq
-      end.flatten
-
-      expect(names.size).to eq(names.uniq.size)
-    end
-  end
-
   describe 'Norway' do
     let(:norway) { ISO3166::Country.search('NO') }
 
@@ -905,6 +1189,24 @@ describe ISO3166::Country do
     end
   end
 
+  describe 'gdpr_compliant?' do
+    let(:united_kigndom) { ISO3166::Country.search('GB') }
+    let(:france) { ISO3166::Country.search('FR') }
+    let(:mexico) { ISO3166::Country.search('MX') }
+
+    it 'should return false for countries without eea_member flag' do
+      expect(mexico.gdpr_compliant?).to be_falsey
+    end
+
+    it 'should return true for countries with eea_member flag set to true' do
+      expect(france.gdpr_compliant?).to be_truthy
+    end
+
+    it 'should return true for UK' do
+      expect(united_kigndom.gdpr_compliant?).to be_truthy
+    end
+  end
+
   describe 'in_esm?' do
     let(:netherlands) { ISO3166::Country.search('NL') }
     let(:switzerland) { ISO3166::Country.search('CH') }
@@ -922,6 +1224,42 @@ describe ISO3166::Country do
     end
   end
 
+  describe 'in_eu_vat?' do
+    let(:netherlands) { ISO3166::Country.search('NL') }
+    let(:guadeloupe) { ISO3166::Country.search('GP') }
+    let(:monaco) { ISO3166::Country.search('MC') }
+
+    it 'should return false for countries without euvat_member or eu_member flag' do
+      expect(country.in_eu_vat?).to be_falsey
+    end
+
+    it 'should return true for countries with eu_member flag set to true' do
+      expect(netherlands.in_eu_vat?).to be_truthy
+    end
+
+    it 'should return false for countries with euvat_member flag set to false' do
+      expect(guadeloupe.in_eu_vat?).to be_falsey
+    end
+
+    it 'should return true for countries with euvat_member flag set to true' do
+      expect(monaco.in_eu_vat?).to be_truthy
+    end
+  end
+
+  describe 'in_un?' do
+    let(:united_states) { ISO3166::Country.search('US') }
+    let(:kiribati) { ISO3166::Country.search('KI') }
+    let(:norfolk_island) { ISO3166::Country.search('NF') }
+
+    it 'should return true for countries with un_member flag set to true' do
+      expect(united_states.in_un?).to be_truthy
+    end
+
+    it 'should return false for countries with un_member flag set to false' do
+      expect(norfolk_island.in_un?).to be_falsey
+    end
+  end
+
   describe 'gec' do
     it 'should return the country\'s GEC code' do
       expect(ISO3166::Country.new('NA').gec).to eql 'WA'
@@ -947,7 +1285,7 @@ describe ISO3166::Country do
 
     it 'should contain all keys for vat_rates' do
       expect(belgium.vat_rates).to be_a(Hash)
-      expect(belgium.vat_rates.keys).to eq(%w(standard reduced super_reduced parking))
+      expect(belgium.vat_rates.keys).to eq(%w[standard reduced super_reduced parking])
     end
 
     it 'should return an array of reduced vat rates' do
@@ -965,13 +1303,25 @@ describe ISO3166::Country do
       expect(ISO3166::Country('us')).to eq country
     end
 
-    it 'should return country if not convertable input given' do
+    it 'should return country if not convertible input given' do
       expect do
         ISO3166::Country(42)
       end.to raise_error(TypeError, /can't convert ([A-z]+) into ISO3166::Country/)
     end
   end
 
+  describe 'Searching by Nationality' do
+     it 'should return Italy for Italian' do
+      expect(ISO3166::Country.find_country_by_nationality('Italian').alpha2).to eq 'IT'
+    end
+    it 'should return Pakistan for Pakistani' do
+      expect(ISO3166::Country.find_country_by_nationality('Pakistani').alpha2).to eq 'PK' 
+    end
+    it 'should return United States for American' do
+      expect(ISO3166::Country.find_country_by_nationality('American').alpha2).to eq 'US'
+    end
+  end
+
   describe 'Added country names to search by' do
     it 'should return country code for Democratic Republic of the Congo' do
       expect(ISO3166::Country.find_country_by_unofficial_names('Democratic Republic of the Congo').alpha2).to eq 'CD'
@@ -1021,18 +1371,132 @@ describe ISO3166::Country do
 
     subject { ISO3166::Country.pluck(*args) }
 
-    it "returns empty arrays" do
+    it 'returns empty arrays' do
       expect(subject.first).to be_empty
       expect(subject.last).to be_empty
     end
 
-    context "when asking for alpha2, alpha3 and iso_short_name" do
-      let(:args) { [:alpha2, :alpha3, :iso_short_name] }
+    context 'when asking for alpha2, alpha3 and iso_short_name' do
+      let(:args) { %i[alpha2 alpha3 iso_short_name] }
 
-      it "returns the correct values" do
-        expect(subject.first).to eq(["AD", "AND", "Andorra"])
-        expect(subject.last).to eq(["ZW", "ZWE", "Zimbabwe"])
+      it 'returns the correct values' do
+        expect(subject.first).to eq(%w[AD AND Andorra])
+        expect(subject.last).to eq(%w[ZW ZWE Zimbabwe])
+      end
+    end
+
+    context 'with invalid attributes' do
+      let(:args) { %i[alpha2 bad_attribute] }
+
+      it 'should raise an error' do
+        expect { subject }.to(
+          raise_error(KeyError, 'key not found: "bad_attribute"')
+        )
+      end
+    end
+
+    context 'with attributes that may be empty' do
+      # https://github.com/countries/countries/issues/925
+      let(:args) { %i[region languages_spoken] }
+
+
+      it 'returns the correct values' do
+        expect(subject.first).to eq(["Europe", ["ca"]])
+        expect(subject.last).to eq(["Africa", ["en", "sn", "nd"]])
       end
     end
   end
+
+  describe 'find_subdivision_by_name' do
+    let(:italy) { ISO3166::Country.new('IT') }
+    let(:napoli) { italy.subdivisions['NA'] }
+
+    before do
+      ISO3166.configuration.locales = %i[pt en]
+      ISO3166::Data.reset
+    end
+
+    it 'should find a subdivision using the official name' do
+      expect(ISO3166::Country.new('IT').find_subdivision_by_name('Napoli')).to eq napoli
+    end
+
+    it 'should find a subdivision using the code' do
+      expect(ISO3166::Country.new('IT').find_subdivision_by_name('NA')).to eq napoli
+    end
+
+    it 'should find a subdivision using a translated name' do
+      expect(ISO3166::Country.new('IT').find_subdivision_by_name('Nápoles')).to eq napoli
+    end
+
+    it 'should prioritize state over district' do
+      country = ISO3166::Country.new('US')
+      expect(country.find_subdivision_by_name('Washington').code).to eq 'WA'
+      code = country.find_subdivision_by_name('Washington DC').code
+      expect(code).to eq 'DC'
+      expect(country.find_subdivision_by_name('District of Columbia').code).to eq 'DC'
+    end
+  end
+
+  describe 'collect_countries_with' do
+    let(:italy) { ISO3166::Country.new('IT') }
+    let(:vatican) { ISO3166::Country.new('VA') }
+    let(:san_marino) { ISO3166::Country.new('SM') }
+    let(:switzerland) { ISO3166::Country.new('CH') }
+    let(:seychelles) { ISO3166::Country.new('SC') }
+
+    it 'defaults to querying alpha2 and returning the countries' do
+      expect(ISO3166::Country.collect_countries_with('IT')).to eq [italy]
+    end
+
+    it 'allows querying by other attributes' do
+      expect(ISO3166::Country.collect_countries_with('🇸🇨', :emoji_flag)).to eq [seychelles]
+      expect(ISO3166::Country.collect_countries_with('it',
+                                                     :languages_spoken)).to eq [switzerland, italy, san_marino, vatican]
+    end
+
+    it 'allows applying a method to the result set' do
+      expect(ISO3166::Country.collect_countries_with('Caribbean', :subregion,
+                                                     :languages_spoken).flatten.uniq).to eq %w[en nl fr es ht]
+    end
+  end
+
+  describe 'collect_likely_countries_by_subdivision_name' do
+    let(:costa_rica) { ISO3166::Country.new('CR') }
+    let(:uruguay) { ISO3166::Country.new('UY') }
+
+    it 'defaults to returning the countries' do
+      expect(ISO3166::Country.collect_likely_countries_by_subdivision_name('San José')).to eq [costa_rica, uruguay]
+    end
+
+    it 'allows applying a method to the result set' do
+      expect(ISO3166::Country.collect_likely_countries_by_subdivision_name('San José',
+                                                                           :iso_short_name)).to eq ['Costa Rica',
+                                                                                                    'Uruguay']
+    end
+  end
+
+  describe '#pluck support' do
+    require 'active_support/core_ext/enumerable'
+
+    it 'supports plucking attributes on an array of countries' do
+      array = [ISO3166::Country['US'], ISO3166::Country['FR']]
+
+      expect(array.pluck(:iso_short_name)).to eq ['United States of America', 'France']
+      expect(array.pluck(:alpha2, :alpha3)).to eq [%w[US USA], %w[FR FRA]]
+    end
+
+    it 'supports plucking methods on an array of countries' do
+      array = [ISO3166::Country['US'], ISO3166::Country['FR']]
+
+      expect(array.pluck(:in_eu?)).to eq [false, true]
+      expect(array.pluck(:in_eu?, :in_eea?)).to eq [[false, false], [true, true]]
+    end
+
+    it 'supports plucking a mix of attributes and methods on an array of countries' do
+      array = [ISO3166::Country['US'], ISO3166::Country['FR']]
+
+      expect(array.pluck(:alpha2, :in_eu?)).to eq [['US', false], ['FR', true]]
+      expect(array.pluck(:in_eea?, :iso_short_name)).to eq [[false, 'United States of America'], [true, 'France']]
+    end
+  end
 end
