@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 describe 'Accessing ISO3166::Country instances data in multiple threads' do
   before do
     if Thread.respond_to?(:report_on_exception)
@@ -12,9 +14,9 @@ describe 'Accessing ISO3166::Country instances data in multiple threads' do
     nthreads = 100
     threads = []
 
-    alpha2_codes = ['us', 'es', 'nl', 'ca', 'de', 'fr', 'mx', 'ru', 'ch', 'jp']
+    alpha2_codes = %w[us es nl ca de fr mx ru ch jp]
 
-    nthreads.times do
+    nthreads.times do |_i|
       threads << Thread.new do
         alpha2_codes.each do |a2|
           country = ISO3166::Country[a2]
@@ -27,20 +29,13 @@ describe 'Accessing ISO3166::Country instances data in multiple threads' do
     threads.map(&:join)
   end
 
-  it "doesn't raise any exceptions when using a mutex" do
+  it "doesn't raise any exceptions" do
     expect { create_countries_threaded }.to_not raise_error
   end
 
-  it "raises NoMethodError when not using a mutex" do
-    allow(ISO3166::Data).to receive(:use_mutex?).and_return(false)
-
-    expect { create_countries_threaded }.to raise_error(NoMethodError)
-  end
-
   after do
     if Thread.respond_to?(:report_on_exception)
       Thread.report_on_exception = @report_on_exception_value
     end
   end
 end
-
